#! /usr/bin/env python3
# -*- coding: utf8 -*-

''' Tests MPS-PICO8 driver functionality '''

import fcntl
import struct
import numpy as np
import os
import picodefs

class DriverInterface(object):
    ''' Interfaces with MPS-PICO8 Linux driver '''


    def __init__(self, filename, debug=True):
        super(DriverInterface, self).__init__()
        self.debug = debug
        self.f = os.open(filename, os.O_RDWR)

        if self.debug:
            print('Opened', filename, 'with fileno', self.f)
            pass

    def __del__(self):
        os.close(self.f)
        if self.debug:
            print('Closed', self.f.name)

    def read(self, nr_samp, print_data=False):
        ''' Reads picoammeter data'''
        if self.debug:
            print('read(', str(nr_samp), ')')

        # nr_samp = nr_samp @ 1MHz 
        nr_samp_bytes = int(nr_samp * 64)      # TLP packet 64 bytes
        buf = os.read(self.f, nr_samp_bytes)   
        b_trans = self.get_b_trans()

        buf = buf[0:b_trans]

        formatStr = '<' + 'ffffffffIIIIIIII' * (len(buf) // 64)
        chs = struct.unpack(formatStr, buf)
        chs = np.array(chs)
        chs = chs.reshape((nr_samp, 16))

        if print_data:
            print(chs, b_trans)

        return chs

    def set_fsamp(self, fsamp):
        ''' Sets picoammeter sampling frequency '''
        ''' fsamp is restricted to the range [1e6, 150e3] Hz '''
        if self.debug:
            print('set_fsamp(', str(fsamp), ')')

        buf = struct.pack('I', int(fsamp))
        fcntl.ioctl(self.f, picodefs.SET_FSAMP, buf)

    def set_range(self, rng):
        ''' Sets picoammeter range '''
        if self.debug:
            print('set_range(', '0b{0:08b}'.format(rng), ')')
        buf = struct.pack('B', rng)
        fcntl.ioctl(self.f, picodefs.SET_RANGE, buf)

    def get_b_trans(self):
        ''' Gets number of bytes transfered in previous read() '''
        buf = fcntl.ioctl(self.f, picodefs.GET_B_TRANS, '    ')
        b_trans = struct.unpack('I', buf)[0]
        if self.debug:
            print('get_b_trans():', str(b_trans))
        return b_trans

    def set_gate_mux(self, sel):
        ''' Sets gate mux '''
        if self.debug:
            print('set_gate_mux(', str(sel), ')')

        buf = struct.pack('I', sel)
        fcntl.ioctl(self.f, picodefs.SET_GATE_MUX, buf)

    def set_conv_mux(self, sel):
        ''' Sets convert mux '''
        if self.debug:
            print('set_conv_mux(', str(sel), ')')

        buf = struct.pack('I', sel)
        fcntl.ioctl(self.f, picodefs.SET_CONV_MUX, buf)

    def set_ring_buf(self, nr_samp):
        ''' Sets number of bytes in ring buffer (pre-trigger condition) '''
        if self.debug:
            print('set_ring_buf(', str(nr_samp), ')')

        buf = struct.pack('I', nr_samp)
        fcntl.ioctl(self.f, picodefs.SET_RING_BUF, buf)

    def set_trigger(self, limit, ch_sel, nr_samp, mode):
        ''' Sets trigger control '''
        if self.debug:
            print('set_trigger(', str(limit), ',', str(nr_samp), ',', str(mode), ')')

        buf = bytes()
        buf += struct.pack('f', limit)
        buf += struct.pack('I', nr_samp)
        buf += struct.pack('I', ch_sel)
        buf += struct.pack('I', mode)

        fcntl.ioctl(self.f, picodefs.SET_TRG, buf)
    
    # Read Threshold Flags
    def get_threshold_flag(self, data, addr):
        ''' Gets threshold flags '''
        buf_in = bytes()
        buf_in += struct.pack('I', data)
        buf_in += struct.pack('I', addr) 
        buf = fcntl.ioctl(self.f, picodefs.GET_THRESHOLD_FLAG, buf_in)
        read_th_flag = struct.unpack('I', buf[0:4])[0]
        if self.debug:
            print('get_threshold_flag():', str(read_th_flag))
        return read_th_flag

    # Read/Write Thresholds
    def set_threshold(self, data, addr):
        ''' Sets thresholds '''
        if self.debug:
            print('set_threshold(', str(data), str(addr), ')')
        buf = bytes()
        buf += struct.pack('f', data)
        buf += struct.pack('I', addr)
        fcntl.ioctl(self.f, picodefs.SET_THRESHOLD, buf)

    def get_threshold(self, data, addr):
        ''' Gets thresholds '''
        buf_in = bytes()
        buf_in += struct.pack('f', data)
        buf_in += struct.pack('I', addr) 
        buf = fcntl.ioctl(self.f, picodefs.GET_THRESHOLD, buf_in)
        read_th = struct.unpack('f', buf[0:4])[0]
        if self.debug:
            print('get_threshold():', str(read_th))
        return read_th

    # Read/Write NSAMP
    def set_nsamp(self, data, addr):
        ''' Sets nsamp '''
        if self.debug:
            print('set_nsamp(', str(data), str(addr), ')')
        buf = bytes()
        buf += struct.pack('I', data)
        buf += struct.pack('I', addr)
        fcntl.ioctl(self.f, picodefs.SET_NSAMP, buf)

    def get_nsamp(self, data, addr):
        ''' Gets nsamp '''
        buf_in = bytes()
        buf_in += struct.pack('I', data)
        buf_in += struct.pack('I', addr) 
        buf = fcntl.ioctl(self.f, picodefs.GET_NSAMP, buf_in)
        read_nsamp = struct.unpack('I', buf[0:4])[0]
        if self.debug:
            print('get_nsamp():', str(read_nsamp))
        return read_nsamp

    def set_mps_config(self, enable, mps_mux):
        ''' Sets mps config '''
        if self.debug:
            print('set_mps_config(', str(enable), str(mps_mux), ')')
        data = int((mps_mux << 1) | enable)
        buf = bytes()
        buf += struct.pack('I', data)
        fcntl.ioctl(self.f, picodefs.SET_MPS_CONFIG, buf)

    def get_mps_config(self, enable, mps_mux):
        ''' Sets mps config '''
        buf_in = bytes()
        buf_in += struct.pack('I', mps_mux)
        buf = fcntl.ioctl(self.f, picodefs.GET_MPS_CONFIG, buf_in)
        read_tmp = struct.unpack('I', buf[0:4])[0]
        enable = read_tmp & 0x1
        mps_mux = (read_tmp >> 1) & 0x7
        if self.debug:
            print('get_mps_config(', str(enable), str(mps_mux), ')')
        return enable, mps_mux
    
    def set_mps_ctrl(self, data):
        ''' Set MPS control register '''
        if self.debug:
            print('set_mps_ctrl(', str(data), ')')
        buf = bytes()
        buf += struct.pack('I', data)
        fcntl.ioctl(self.f, picodefs.SET_MPS_CTRL, buf)

    def set_trg_acq_samples(self, acq_samples, delay_samples):
        ''' Sets acq and delay samples '''
        if self.debug:
            print('set_trg_acq_samples( acq:', str(acq_samples), 'delay:', str(delay_samples), ')')
        buf = struct.pack('I', acq_samples)
        fcntl.ioctl(self.f, picodefs.SET_TRG_NRSAMP, buf)
        buf = struct.pack('I', delay_samples)
        fcntl.ioctl(self.f, picodefs.SET_TRG_DELAY, buf)

    def get_trg_acq_samples(self, data):
        ''' Gets trigger acq samples '''
        buf_in = bytes()
        buf_in += struct.pack('I', data)
        buf = fcntl.ioctl(self.f, picodefs.GET_TRG_NRSAMP, buf_in)
        read_nsamp = struct.unpack('I', buf[0:4])[0]
        if self.debug:
            print('get_trg_acq_samples():', str(read_nsamp))
        return read_nsamp
    
    def get_trg_delay_samples(self, data):
        ''' Gets trigger acq samples '''
        buf_in = bytes()
        buf_in += struct.pack('I', data)
        buf = fcntl.ioctl(self.f, picodefs.GET_TRG_DELAY, buf_in)
        read_nsamp = struct.unpack('I', buf[0:4])[0]
        if self.debug:
            print('get_trg_delay_samples():', str(read_nsamp))
        return read_nsamp

    def set_acq_mode(self, mode, acq_samples, delay_samples):
        ''' Sets acq mode '''
        if self.debug:
            print('set_acq_mode( mode:', str(mode), 'acq:', str(acq_samples), 'delay:', str(delay_samples), ')')
        if (mode == 0):  # Infinite mode
            buf = struct.pack('I', mode)
            fcntl.ioctl(self.f, picodefs.SET_MPS_CTRL, buf)
        else: # Trigger mode
            buf = struct.pack('I', acq_samples)
            fcntl.ioctl(self.f, picodefs.SET_TRG_NRSAMP, buf)
            buf = struct.pack('I', delay_samples)
            fcntl.ioctl(self.f, picodefs.SET_TRG_DELAY, buf)
            buf = struct.pack('I', mode)
            fcntl.ioctl(self.f, picodefs.SET_MPS_CTRL, buf)

    def get_accumulator_data(self, data, addr):
        ''' Gets Accumulator data in Trigger Mode '''
        buf_in = bytes()
        buf_in += struct.pack('f', data)
        buf_in += struct.pack('I', addr) 
        buf = fcntl.ioctl(self.f, picodefs.GET_TRG_ACC_DATA, buf_in)
        read_data = struct.unpack('f', buf[0:4])[0]
        if self.debug:
            print('get_accu_data():', str(read_data))
        return read_data

    def get_trg_cntr_1M(self, data):
        ''' Gets Trigger Counter 1M '''
        buf_in = bytes()
        buf_in += struct.pack('I', data)
        buf = fcntl.ioctl(self.f, picodefs.GET_TRG_CNTR_1M, buf_in)
        read_cntr = struct.unpack('I', buf[0:4])[0]
        if self.debug:
            print('get_trg_cntr_1M():', str(read_cntr))
        return read_cntr

    def get_trg_cntr_1k(self, data):
        ''' Gets Trigger Counter 1M '''
        buf_in = bytes()
        buf_in += struct.pack('I', data)
        buf = fcntl.ioctl(self.f, picodefs.GET_TRG_CNTR_1K, buf_in)
        read_cntr = struct.unpack('I', buf[0:4])[0]
        if self.debug:
            print('get_trg_cntr_1k():', str(read_cntr))
        return read_cntr